@@ -1,6 +1,6 @@
 ;;; emms-browser.el --- a track browser supporting covers and filtering  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2006, 2007, 2008, 2009  Free Software Foundation, Inc.
+;; Copyright (C) 2006-2021  Free Software Foundation, Inc.
 
 ;; Author: Damien Elmes <emacs@repose.cx>
 ;; Keywords: emms, mp3, mpeg, multimedia
@@ -190,10 +190,12 @@
 ;; artist->track instead of artist->album->track when you switch to
 ;; the 'singles' filter.
 
-;; (defadvice emms-browser-next-mapping-type
-;;                                 (after no-album (current-mapping))
-;;   (when (eq ad-return-value 'info-album)
-;;     (setq ad-return-value 'info-title)))
+;; (advice-add 'emms-browser-next-mapping-type :around #'my-emms--types)
+;; (defun my-emms--types (orig-fun &rest args)
+;;   (let ((type (apply orig-fun args)))
+;;     (if (eq type 'info-album)
+;;         'info-title)
+;;       type)))
 
 ;; (defun toggle-album-display ()
 ;;   (if (string= emms-browser-current-filter-name "singles")
@@ -286,6 +288,7 @@
 (require 'cl-lib)
 (require 'emms)
 (require 'emms-cache)
+(require 'emms-volume)
 (require 'emms-source-file)
 (require 'emms-playlist-sort)
 (require 'sort)
@@ -307,117 +310,94 @@
 
 (defcustom emms-browser-default-browse-type
   'info-artist
-  "*The default browsing mode."
-  :group 'emms-browser
-  :type 'function)
-
-(defcustom emms-browser-make-name-function
-  'emms-browser-make-name-standard
-  "*A function to make names for entries and subentries.
-Overriding this function allows you to customise how various elements
-are displayed. It is called with two arguments - track and type."
-  :group 'emms-browser
+  "The default browsing mode."
   :type 'function)
 
 (defcustom emms-browser-get-track-field-function
-  'emms-browser-get-track-field-albumartist
-  "*A function to get an element from a track.
+  #'emms-browser-get-track-field-albumartist
+  "A function to get an element from a track.
 Change this to customize the way data is organized in the
 browser. For example,
 `emms-browser-get-track-field-use-directory-name' uses the
 directory name to determine the artist. This means that
 soundtracks, compilations and so on don't populate the artist
 view with lots of 1-track elements."
-  :group 'emms-browser
   :type '(choice (function :tag "Sort by album-artist" emms-browser-get-track-field-albumartist)
                  (function :tag "Simple" emms-browser-get-track-field-simple)))
 
 (defcustom emms-browser-covers
   '("cover_small" "cover_med" "cover_large")
-  "*Control how cover images are found.
+  "Control how cover images are found.
 Can be either a list of small, medium and large images (large
 currently not used), a function which takes a directory and one
 of the symbols `small', `medium' or `large', and should return a
 path to the cover, or nil to turn off cover loading."
-  :group 'emms-browser
   :type '(choice list function boolean))
 
 (defcustom emms-browser-covers-file-extensions
   '("jpg" "jpeg" "png" "gif" "bmp")
-  "*File extensions accepted for `emms-browser-covers'.
+  "File extensions accepted for `emms-browser-covers'.
 Should be a list of extensions as strings.  Should be set before
 emms-browser is required."
-  :group 'emms-browser
   :type '(repeat (string :tag "Extension")))
 
 (defconst emms-browser--covers-filename nil
   "*List of potential cover art names.")
 
 (defcustom emms-browser-default-covers nil
-  "*A list of default images to use if a cover isn't found."
-  :group 'emms-browser
+  "A list of default images to use if a cover isn't found."
   :type 'list)
 
 (defcustom emms-browser-comparison-test
   (if (fboundp 'define-hash-table-test)
       'case-fold
     'equal)
-  "*A method for comparing entries in the cache.
+  "A method for comparing entries in the cache.
 The default is to compare case-insensitively."
-  :group 'emms-browser
   :type 'symbol)
 
 (defcustom emms-browser-track-sort-function
-  'emms-sort-natural-order-less-p
-  "*How to sort tracks in the browser.
+  #'emms-sort-natural-order-less-p
+  "How to sort tracks in the browser.
 Ues nil for no sorting."
-  :group 'emms-browser
   :type 'function)
 
 (defcustom emms-browser-alpha-sort-function
-  (if (functionp 'string-collate-lessp) 'string-collate-lessp 'string<)
-  "*How to sort artists/albums/etc. in the browser.
+  (if (functionp 'string-collate-lessp) #'string-collate-lessp #'string<)
+  "How to sort artists/albums/etc. in the browser.
 Use nil for no sorting."
-  :group 'emms-browser
   :type 'function)
 
 (defcustom emms-browser-album-sort-function
-  'emms-browser-sort-by-year-or-name
-  "*How to sort artists/albums/etc. in the browser.
+  #'emms-browser-sort-by-year-or-name
+  "How to sort artists/albums/etc. in the browser.
 Use nil for no sorting."
-  :group 'emms-browser
   :type 'function)
 
 (defcustom emms-browser-show-display-hook nil
-  "*Hooks to run when starting or switching to a browser buffer."
-  :group 'emms-browser
+  "Hooks to run when starting or switching to a browser buffer."
   :type 'hook)
 
 (defcustom emms-browser-hide-display-hook nil
-  "*Hooks to run when burying or removing a browser buffer."
-  :group 'emms-browser
+  "Hooks to run when burying or removing a browser buffer."
   :type 'hook)
 
 (defcustom emms-browser-tracks-added-hook nil
-  "*Hooks to run when tracks are added to the playlist."
-  :group 'emms-browser
+  "Hooks to run when tracks are added to the playlist."
   :type 'hook)
 
 (defcustom emms-browser-filter-tracks-hook nil
-  "*Given a track, return t if the track should be ignored."
-  :group 'emms-browser
+  "Given a track, return t if the track should be ignored."
   :type 'hook)
 
 (defcustom emms-browser-filter-changed-hook nil
-  "*Hook run after the filter has changed."
-  :group 'emms-browser
+  "Hook run after the filter has changed."
   :type 'hook)
 
 (defcustom emms-browser-delete-files-hook nil
-  "*Hook run after files have been deleted.
+  "Hook run after files have been deleted.
 This hook can be used to clean up extra files, such as album covers.
 Called once for each directory."
-  :group 'emms-browser
   :type 'hook)
 
 (defvar emms-browser-buffer nil
@@ -445,55 +425,55 @@ Called once for each directory."
 
 (defvar emms-browser-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map (kbd "q") 'emms-browser-bury-buffer)
-    (define-key map (kbd "/") 'emms-isearch-buffer)
-    (define-key map (kbd "r") 'emms-browser-goto-random)
-    (define-key map (kbd "n") 'next-line)
-    (define-key map (kbd "p") 'previous-line)
-    (define-key map (kbd "C") 'emms-browser-clear-playlist)
-    (define-key map (kbd "?") 'describe-mode)
-    (define-key map (kbd "C-/") 'emms-playlist-mode-undo)
-    (define-key map (kbd "SPC") 'emms-browser-toggle-subitems)
-    (define-key map (kbd "^") 'emms-browser-move-up-level)
-    (define-key map (kbd "RET") 'emms-browser-add-tracks)
-    (define-key map (kbd "<C-return>") 'emms-browser-add-tracks-and-play)
-    (define-key map (kbd "C-j") 'emms-browser-add-tracks-and-play)
-    (define-key map (kbd "<tab>") 'emms-browser-next-non-track)
-    (define-key map (kbd "<backtab>") 'emms-browser-prev-non-track)
-    (define-key map (kbd "d") 'emms-browser-view-in-dired)
-    (define-key map (kbd "D") 'emms-browser-delete-files)
-    (define-key map (kbd "E") 'emms-browser-expand-all)
-    (define-key map (kbd "1") 'emms-browser-collapse-all)
-    (define-key map (kbd "2") 'emms-browser-expand-to-level-2)
-    (define-key map (kbd "3") 'emms-browser-expand-to-level-3)
-    (define-key map (kbd "4") 'emms-browser-expand-to-level-4)
-    (define-key map (kbd "b 1") 'emms-browse-by-artist)
-    (define-key map (kbd "b 2") 'emms-browse-by-album)
-    (define-key map (kbd "b 3") 'emms-browse-by-genre)
-    (define-key map (kbd "b 4") 'emms-browse-by-year)
-    (define-key map (kbd "b 5") 'emms-browse-by-composer)
-    (define-key map (kbd "b 6") 'emms-browse-by-performer)
-    (define-key map (kbd "s a") 'emms-browser-search-by-artist)
-    (define-key map (kbd "s c") 'emms-browser-search-by-composer)
-    (define-key map (kbd "s p") 'emms-browser-search-by-performer)
-    (define-key map (kbd "s A") 'emms-browser-search-by-album)
-    (define-key map (kbd "s t") 'emms-browser-search-by-title)
-    (define-key map (kbd "s s") 'emms-browser-search-by-names)
-    (define-key map (kbd "W A w") 'emms-browser-lookup-artist-on-wikipedia)
-    (define-key map (kbd "W C w") 'emms-browser-lookup-composer-on-wikipedia)
-    (define-key map (kbd "W P w") 'emms-browser-lookup-performer-on-wikipedia)
-    (define-key map (kbd "W a w") 'emms-browser-lookup-album-on-wikipedia)
-    (define-key map (kbd ">") 'emms-browser-next-filter)
-    (define-key map (kbd "<") 'emms-browser-previous-filter)
-    (define-key map (kbd "+") 'emms-volume-raise)
-    (define-key map (kbd "-") 'emms-volume-lower)
+    (define-key map (kbd "q") #'emms-browser-bury-buffer)
+    (define-key map (kbd "/") #'emms-isearch-buffer)
+    (define-key map (kbd "r") #'emms-browser-goto-random)
+    (define-key map (kbd "n") #'next-line)
+    (define-key map (kbd "p") #'previous-line)
+    (define-key map (kbd "C") #'emms-browser-clear-playlist)
+    (define-key map (kbd "?") #'describe-mode)
+    (define-key map (kbd "C-/") #'emms-playlist-mode-undo)
+    (define-key map (kbd "SPC") #'emms-browser-toggle-subitems)
+    (define-key map (kbd "^")   #'emms-browser-move-up-level)
+    (define-key map (kbd "RET") #'emms-browser-add-tracks)
+    (define-key map (kbd "<C-return>") #'emms-browser-add-tracks-and-play)
+    (define-key map (kbd "C-j") #'emms-browser-add-tracks-and-play)
+    (define-key map (kbd "<tab>") #'emms-browser-next-non-track)
+    (define-key map (kbd "<backtab>") #'emms-browser-prev-non-track)
+    (define-key map (kbd "d") #'emms-browser-view-in-dired)
+    (define-key map (kbd "D") #'emms-browser-delete-files)
+    (define-key map (kbd "E") #'emms-browser-expand-all)
+    (define-key map (kbd "1") #'emms-browser-collapse-all)
+    (define-key map (kbd "2") #'emms-browser-expand-to-level-2)
+    (define-key map (kbd "3") #'emms-browser-expand-to-level-3)
+    (define-key map (kbd "4") #'emms-browser-expand-to-level-4)
+    (define-key map (kbd "b 1") #'emms-browse-by-artist)
+    (define-key map (kbd "b 2") #'emms-browse-by-album)
+    (define-key map (kbd "b 3") #'emms-browse-by-genre)
+    (define-key map (kbd "b 4") #'emms-browse-by-year)
+    (define-key map (kbd "b 5") #'emms-browse-by-composer)
+    (define-key map (kbd "b 6") #'emms-browse-by-performer)
+    (define-key map (kbd "s a") #'emms-browser-search-by-artist)
+    (define-key map (kbd "s c") #'emms-browser-search-by-composer)
+    (define-key map (kbd "s p") #'emms-browser-search-by-performer)
+    (define-key map (kbd "s A") #'emms-browser-search-by-album)
+    (define-key map (kbd "s t") #'emms-browser-search-by-title)
+    (define-key map (kbd "s s") #'emms-browser-search-by-names)
+    (define-key map (kbd "W A w") #'emms-browser-lookup-artist-on-wikipedia)
+    (define-key map (kbd "W C w") #'emms-browser-lookup-composer-on-wikipedia)
+    (define-key map (kbd "W P w") #'emms-browser-lookup-performer-on-wikipedia)
+    (define-key map (kbd "W a w") #'emms-browser-lookup-album-on-wikipedia)
+    (define-key map (kbd ">") #'emms-browser-next-filter)
+    (define-key map (kbd "<") #'emms-browser-previous-filter)
+    (define-key map (kbd "+") #'emms-volume-raise)
+    (define-key map (kbd "-") #'emms-volume-lower)
     map)
   "Keymap for `emms-browser-mode'.")
 
 (defvar emms-browser-search-mode-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map emms-browser-mode-map)
-    (define-key map (kbd "q") 'emms-browser-kill-search)
+    (define-key map (kbd "q") #'emms-browser-kill-search)
     map)
   "Keymap for `emms-browser-mode'.")
 
@@ -502,19 +482,20 @@ Called once for each directory."
 ;; --------------------------------------------------
 
 (eval-and-compile
-  (if (fboundp 'with-selected-window)
-      (defalias 'emms-browser-with-selected-window 'with-selected-window)
+  (if (fboundp 'with-selected-window)   ;Emacs-22
+      (defalias 'emms-browser-with-selected-window #'with-selected-window)
     (defmacro emms-browser-with-selected-window (window &rest body)
       (ignore window)
       ;; this emulates the behavior introduced earlier, though it
       ;; might be best to do something with `window'
       `(save-selected-window ,body)))
   (put 'emms-browser-with-selected-window 'lisp-indent-function 1)
-  (put 'emms-browser-with-selected-window 'edebug-form-spec '(form body))
+  (put 'emms-browser-with-selected-window 'edebug-form-spec '(form body)))
 
-  (if (fboundp 'run-mode-hooks)
-      (defalias 'emms-browser-run-mode-hooks 'run-mode-hooks)
-    (defalias 'emms-browser-run-mode-hooks 'run-hooks)))
+(defalias 'emms-browser-run-mode-hooks
+  (if (fboundp 'run-mode-hooks)         ;Emacs-22
+      #'run-mode-hooks
+    #'run-hooks))
 
 ;; --------------------------------------------------
 ;; General mode setup
@@ -554,7 +535,7 @@ example function is `emms-browse-by-artist'."
 (defun emms-browser-mode (&optional no-update)
   "A major mode for the Emms browser.
 \\{emms-browser-mode-map}"
-      ;; create a new buffer
+  ;; create a new buffer
   (interactive)
 
   (use-local-map emms-browser-mode-map)
@@ -612,10 +593,10 @@ If a browser search exists, return it."
   "Create an interactive function emms-browse-by-NAME."
   (let ((funname (intern (concat "emms-browse-by-" name)))
         (funcdesc (concat "Browse by " name ".")))
-  `(defun ,funname ()
-     ,funcdesc
-     (interactive)
-     (emms-browse-by ,type))))
+    `(defun ,funname ()
+       ,funcdesc
+       (interactive)
+       (emms-browse-by ,type))))
 
 (defun emms-browse-by (type)
   "Render a top level buffer based on TYPE."
@@ -719,7 +700,7 @@ For 'info-year TYPE, use 'info-originalyear, 'info-originaldate and
            (sort-subr nil
                       #'forward-line #'end-of-line
                       (lambda () (buffer-substring-no-properties
-                             (line-beginning-position) (line-end-position)))
+				  (line-beginning-position) (line-end-position)))
                       nil
                       emms-browser-alpha-sort-function))
        (sort-lines nil (point-min) (point-max))))))
@@ -816,14 +797,14 @@ artist1 -> album1 -> *track* 1.."
   (let ((key (car entry))
         (track (cadr entry))
         artist title) ;; only the first track
-  (cond
-   ((eq type 'info-title)
-    (setq artist (emms-track-get track 'info-artist))
-    (setq title (emms-track-get track 'info-title))
-    (if (not (and artist title))
-        key
-      (concat artist " - " title)))
-   (t key))))
+    (cond
+     ((eq type 'info-title)
+      (setq artist (emms-track-get track 'info-artist))
+      (setq title (emms-track-get track 'info-title))
+      (if (not (and artist title))
+          key
+	(concat artist " - " title)))
+     (t key))))
 
 (defun emms-browser-track-number (track)
   "Return a string representation of a track number.
@@ -883,7 +864,7 @@ Items with no metadata for TYPE will be placed in 'misc'"
     (dolist (track tracks)
       (setq key (emms-browser-get-track-field track type))
       (when (eq type 'info-title)
-          ;; try and make every track unique
+        ;; try and make every track unique
         (setq tracknum (emms-browser-track-number track))
         (if (string= tracknum "")
             (setq key (file-name-nondirectory
@@ -935,6 +916,7 @@ TYPE is the metadata to make the alist by - eg. if it's
 This sorting predicate will compare the cadr of each entry.
 SORT-FUNC should be a playlist sorting predicate like
 `emms-playlist-sort-by-natural-order'."
+  (declare (debug t))
   `(lambda (a b)
      (funcall ,sort-func (cadr a) (cadr b))))
 
@@ -943,6 +925,7 @@ SORT-FUNC should be a playlist sorting predicate like
 This sorting predicate will compare the car of each entry.
 SORT-FUNC should be a playlist sorting predicate like
 `emms-playlist-sort-by-natural-order'."
+  (declare (debug t))
   `(lambda (a b)
      (funcall ,sort-func (car a) (car b))))
 
@@ -951,7 +934,7 @@ SORT-FUNC should be a playlist sorting predicate like
 Uses `emms-browser-track-sort-function'."
   (if emms-browser-track-sort-function
       (sort alist (emms-browser-sort-cadr
-                  emms-browser-track-sort-function))
+                   emms-browser-track-sort-function))
     alist))
 
 (defun emms-browser-sort-by-name (alist)
@@ -959,7 +942,7 @@ Uses `emms-browser-track-sort-function'."
 Uses `emms-browser-alpha-sort-function'."
   (if emms-browser-alpha-sort-function
       (sort alist (emms-browser-sort-car
-                  emms-browser-alpha-sort-function))
+                   emms-browser-alpha-sort-function))
     alist))
 
 (defun emms-browser-sort-by-year-or-name (alist)
@@ -1425,12 +1408,12 @@ tracks from point, it does not delete files."
     (when (numberp delete) (setq delete nil))
     (when delete
       (save-mark-and-excursion
-       (when (use-region-p) (goto-char start))
-       (let ((lines (min count (- (line-number-at-pos (point-max)) (line-number-at-pos (point))))))
-         (dotimes (_ lines)
-           ;; TODO: Test this!
-           (setq tracks (append tracks (emms-browser-tracks-at-point)))
-           (forward-line))))
+	(when (use-region-p) (goto-char start))
+	(let ((lines (min count (- (line-number-at-pos (point-max)) (line-number-at-pos (point))))))
+          (dotimes (_ lines)
+            ;; TODO: Test this!
+            (setq tracks (append tracks (emms-browser-tracks-at-point)))
+            (forward-line))))
       (unless (yes-or-no-p
                (format "Really permanently delete these %d tracks? " (length tracks)))
         (error "Cancelled!"))
@@ -1502,7 +1485,6 @@ tracks from point, it does not delete files."
 (defcustom emms-browser-switch-to-playlist-on-add
   nil
   "Whether to switch to to the playlist after adding files."
-  :group 'emms-browser
   :type 'boolean)
 
 ;;;###autoload
@@ -1512,53 +1494,53 @@ Toggle between selecting browser, playlist or hiding both. Tries
 to behave sanely if the user has manually changed the window
 configuration."
   (interactive)
-  (add-to-list 'emms-browser-show-display-hook
-               'emms-browser-display-playlist)
-  (add-to-list 'emms-browser-hide-display-hook
-               'emms-browser-hide-linked-window)
+  (add-hook 'emms-browser-show-display-hook
+            #'emms-browser-display-playlist)
+  (add-hook 'emms-browser-hide-display-hook
+            #'emms-browser-hide-linked-window)
   ;; switch to the playlist window when adding tracks?
-  (add-to-list 'emms-browser-tracks-added-hook
-               (lambda (start-of-tracks) (interactive)
-                 (let (playlist-window)
-                   (when emms-browser-switch-to-playlist-on-add
-                     (emms-smart-browse))
-                   ;; center on the first added track/group name
-                   (when
-                       (setq playlist-window
-                             (emms-browser-get-linked-window))
-                     (emms-browser-with-selected-window
-                         playlist-window
-                       (goto-char start-of-tracks)
-                       (recenter '(4)))))))
+  (add-hook 'emms-browser-tracks-added-hook
+            (lambda (start-of-tracks) (interactive)
+              (let (playlist-window)
+                (when emms-browser-switch-to-playlist-on-add
+                  (emms-smart-browse))
+                ;; center on the first added track/group name
+                (when
+                    (setq playlist-window
+                          (emms-browser-get-linked-window))
+                  (emms-browser-with-selected-window
+                      playlist-window
+                    (goto-char start-of-tracks)
+                    (recenter '(4)))))))
   (let (wind)
-  (cond
-   ((eq major-mode 'emms-browser-mode)
-    (setq wind (emms-browser-get-linked-window))
-    ;; if the playlist window is visible, select it
-    (if wind
-        (select-window wind)
-      ;; otherwise display and select it
-      (select-window (emms-browser-display-playlist))))
-   ((eq major-mode 'emms-playlist-mode)
-    (setq wind (emms-browser-get-linked-window))
-    ;; if the playlist window is selected, and the browser is visible,
-    ;; hide both
-    (if wind
-        (progn
+    (cond
+     ((eq major-mode 'emms-browser-mode)
+      (setq wind (emms-browser-get-linked-window))
+      ;; if the playlist window is visible, select it
+      (if wind
           (select-window wind)
-          (emms-browser-bury-buffer)
-          ;; After a browser search, the following buffer could be the
-          ;; unfiltered browser, which we want to bury as well.  We don't want
-          ;; to call `emms-browser-hide-display-hook' for this one so we bury it
-          ;; directly.
-          (when (eq major-mode 'emms-browser-mode)
-            (bury-buffer)))
-      ;; otherwise bury both
-      (bury-buffer)
-      (emms-browser-hide-linked-window)))
-   (t
-    ;; show both
-    (emms-browser)))))
+	;; otherwise display and select it
+	(select-window (emms-browser-display-playlist))))
+     ((eq major-mode 'emms-playlist-mode)
+      (setq wind (emms-browser-get-linked-window))
+      ;; if the playlist window is selected, and the browser is visible,
+      ;; hide both
+      (if wind
+          (progn
+            (select-window wind)
+            (emms-browser-bury-buffer)
+            ;; After a browser search, the following buffer could be the
+            ;; unfiltered browser, which we want to bury as well.  We don't want
+            ;; to call `emms-browser-hide-display-hook' for this one so we bury it
+            ;; directly.
+            (when (eq major-mode 'emms-browser-mode)
+              (bury-buffer)))
+	;; otherwise bury both
+	(bury-buffer)
+	(emms-browser-hide-linked-window)))
+     (t
+      ;; show both
+      (emms-browser)))))
 
 (defun emms-browser-get-linked-buffer ()
   "Return linked buffer (eg browser if playlist is selected."
@@ -1888,11 +1870,11 @@ If > album level, most of the track data will not make sense."
          (name (cond
                 ((or (eq type 'info-year)
                      (eq type 'info-genre)) "year/genre")
-                 ((eq type 'info-artist) "artist")
-                 ((eq type 'info-composer) "composer")
-                 ((eq type 'info-performer) "performer")
-                 ((eq type 'info-album) "album")
-                 ((eq type 'info-title) "track"))))
+                ((eq type 'info-artist) "artist")
+                ((eq type 'info-composer) "composer")
+                ((eq type 'info-performer) "performer")
+                ((eq type 'info-album) "album")
+                ((eq type 'info-title) "track"))))
     (intern
      (concat "emms-browser-" name "-face"))))
 
@@ -2015,8 +1997,7 @@ the text that it generates."
          (t (:background ,dark-col)))
        ,(concat "Face for "
                 name
-                " in a browser/playlist buffer.")
-       :group 'emms-browser-mode)))
+                " in a browser/playlist buffer."))))
 
 (emms-browser-make-face "year/genre" "#aaaaff" "#444477" 1.5)
 (emms-browser-make-face "artist"     "#aaaaff" "#444477" 1.3)
@@ -2037,11 +2018,11 @@ the text that it generates."
 This:
  - defines an interactive function M-x emms-browser-show-NAME.
  - defines a variable emms-browser-filter-NAME of (name . func).
- - adds the filter to emms-browser-filters."
+ - adds the filter to `emms-browser-filters'."
   (let ((funcnam (intern (concat "emms-browser-show-" name)))
         (var  (intern (concat "emms-browser-filter-" name)))
         (desc (concat "Filter the cache using rule '"
-                       name "'")))
+                      name "'")))
     `(progn
        (defvar ,var nil ,desc)
        (setq ,var (cons ,name ,func))
@@ -2068,7 +2049,7 @@ This does not refresh the current buffer."
   "Redisplay with the next filter."
   (interactive)
   (let* ((list (if reverse
-                  (reverse emms-browser-filters)
+                   (reverse emms-browser-filters)
                  emms-browser-filters))
          (key emms-browser-current-filter-name)
          (next (cadr (member (assoc key list) list))))
@@ -2082,30 +2063,31 @@ This does not refresh the current buffer."
   (interactive)
   (emms-browser-next-filter t))
 
-(defun emms-browser-filter-only-dir (path)
-  "Generate a function which checks if a track is in path.
-If the track is not in path, return t."
-  `(lambda (track)
-     (not (string-match ,(concat "^" (expand-file-name path))
-                        (emms-track-get track 'name)))))
+(defun emms-browser-filter-only-dir (dirname)
+  "Generate a function which checks if a track is in DIRNAME.
+If the track is not in DIRNAME, return t."
+  (let ((re (concat "^" (expand-file-name dirname))))
+    (lambda (track)
+      (not (string-match re (emms-track-get track 'name))))))
 
 (defun emms-browser-filter-only-type (type)
   "Generate a function which checks a track's type.
 If the track is not of TYPE, return t."
-  `(lambda (track)
-     (not (eq (quote ,type) (emms-track-get track 'type)))))
+  (lambda (track)
+    (not (eq type (emms-track-get track 'type)))))
 
 ;; seconds in a day (* 60 60 24) = 86400
 (defun emms-browser-filter-only-recent (days)
   "Show only tracks played within the last number of DAYS."
-  `(lambda (track)
-     (let ((min-date (time-subtract
-                      (current-time)
-                      (seconds-to-time (* ,days 86400))))
-           last-played)
-       (not (and (setq last-played
-                       (emms-track-get track 'last-played nil))
-                 (time-less-p min-date last-played))))))
+  (lambda (track)
+    (let ((min-date (time-subtract
+                     (current-time)
+                     (seconds-to-time (* days 86400))))
+          last-played)
+      (not (and (setq last-played
+                      (emms-track-get track 'last-played nil))
+                (time-less-p min-date last-played))))))
+
 
 ;; TODO: Add function to clear the cache from thumbnails that have no associated
 ;; cover folders.  This is especially useful in case the music library path
