@@ -1,6 +1,6 @@
 ;;; emms-player-mpv.el --- mpv support for EMMS  -*- lexical-binding: t; -*-
 ;;
-;; Copyright (C) 2018 Free Software Foundation, Inc.
+;; Copyright (C) 2018-2021 Free Software Foundation, Inc.
 
 ;; Authors: Mike Kazantsev <mk.fraggod@gmail.com>
 
@@ -60,26 +60,30 @@
 
 (require 'emms)
 (require 'emms-player-simple)
+(require 'emms-playing-time)
 (require 'json)
 (require 'cl-lib)
 
 
+(defgroup emms-player-mpv nil
+  "EMMS player for mpv."
+  :group 'emms-player
+  :prefix "emms-player-mpv-")
+
 (defcustom emms-player-mpv
   (emms-player
    #'emms-player-mpv-start
    #'emms-player-mpv-stop
    #'emms-player-mpv-playable-p)
   "*Parameters for mpv player."
-  :type '(cons symbol alist)
-  :group 'emms-player-mpv)
+  :type '(cons symbol alist))
 
 (emms-player-set emms-player-mpv 'regex
                  (apply #'emms-player-simple-regexp emms-player-base-format-list))
 
 (defcustom emms-player-mpv-command-name "mpv"
   "mpv binary to use. Can be absolute path or just binary name."
-  :type 'file
-  :group 'emms-player-mpv)
+  :type 'file)
 
 (defcustom emms-player-mpv-parameters
   '("--quiet" "--really-quiet" "--no-audio-display")
@@ -91,16 +95,14 @@ Note that unless --no-config option is specified here,
 mpv will also use options from its configuration files.
 For mpv binary path, see `emms-player-mpv-command-name'."
   :type '(choice (repeat :tag "List of mpv arguments" string)
-                 function)
-  :group 'emms-player-mpv)
+                 function))
 
 (defcustom emms-player-mpv-environment ()
   "List of extra environment variables (\"VAR=value\" strings) to pass on to mpv process.
 These are added on top of `process-environment' by default.
 Adding nil as an element to this list will discard emacs
 `process-environment' and only pass variables that are specified in the list."
-  :type '(repeat (choice string (const :tag "Start from blank environment" nil)))
-  :group 'emms-player-mpv)
+  :type '(repeat (choice string (const :tag "Start from blank environment" nil))))
 
 (defcustom emms-player-mpv-ipc-method nil
   "Switch for which IPC method to use with mpv.
@@ -113,16 +115,14 @@ support for various feedback and metadata options from mpv."
           (const :tag "Auto-detect from mpv --version" nil)
           (const :tag "Use --input-ipc-server JSON IPC (v0.17.0 2016-04-11)" ipc-server)
           (const :tag "Use --input-unix-socket JSON IPC (v0.7.0 2014-10-16)" unix-socket)
-          (const :tag "Use --input-file FIFO (any mpv version)" file))
-  :group 'emms-player-mpv)
+          (const :tag "Use --input-file FIFO (any mpv version)" file)))
 
 (defcustom emms-player-mpv-ipc-socket
   (concat (file-name-as-directory emms-directory)
           "mpv-ipc.sock")
   "Unix IPC socket or FIFO to use with mpv --input-* options,
 depending on `emms-player-mpv-ipc-method' value and/or mpv version."
-  :type 'file
-  :group 'emms-player-mpv)
+  :type 'file)
 
 (defvar emms-player-mpv-ipc-proc nil) ; to avoid warnings while keeping useful defs at the top
 
@@ -130,7 +130,8 @@ depending on `emms-player-mpv-ipc-method' value and/or mpv version."
   "Update track duration when played by mpv.
 Uses `emms-player-mpv-event-functions' hook."
   :type 'boolean
-  :set (lambda (_sym value)
+  :set (lambda (sym value)
+         (set-default-toplevel-value sym value)
          (run-at-time 0.1 nil
                       (lambda (value)
                         (if value
@@ -140,15 +141,15 @@ Uses `emms-player-mpv-event-functions' hook."
                           (remove-hook
                            'emms-player-mpv-event-functions
                            #'emms-player-mpv-info-duration-event-func)))
-                      value))
-  :group 'emms-player-mpv)
+                      value)))
 
 (defcustom emms-player-mpv-update-metadata nil
   "Update track info (artist, album, name, etc) from mpv events, when it is played.
 This allows to dynamically update stream info from ICY tags, for example.
 Uses `emms-player-mpv-event-connect-hook' and `emms-player-mpv-event-functions' hooks."
   :type 'boolean
-  :set (lambda (_sym value)
+  :set (lambda (sym value)
+         (set-default-toplevel-value sym value)
          (run-at-time 0.1 nil
                       (lambda (value)
                         (if value
@@ -168,8 +169,7 @@ Uses `emms-player-mpv-event-connect-hook' and `emms-player-mpv-event-functions'
                             (remove-hook
                              'emms-player-mpv-event-functions
                              #'emms-player-mpv-info-meta-event-func))))
-                      value))
-  :group 'emms-player-mpv)
+                      value)))
 
 
 (defvar emms-player-mpv-proc nil
@@ -181,7 +181,7 @@ if it refuses to exit cleanly on `emms-player-mpv-proc-stop'.")
 
 
 (defvar emms-player-mpv-ipc-proc nil
-  "Unix socket process that communicates with running `emms-player-mpv-proc' instance.")
+  "Unix socket network process connected to running `emms-player-mpv-proc' instance.")
 
 (defvar emms-player-mpv-ipc-buffer " *emms-player-mpv-ipc*"
   "Buffer to associate with `emms-player-mpv-ipc-proc' socket/pipe process.")
@@ -193,33 +193,40 @@ if it refuses to exit cleanly on `emms-player-mpv-proc-stop'.")
   "List of delays before initiating socket connection for new mpv process.")
 
 (defvar emms-player-mpv-ipc-connect-command nil
-  "JSON command for `emms-player-mpv-ipc-sentinel' to run as soon as it connects to mpv.
-I.e. last command that either initiated connection or was used while connecting to mpv.
+  "JSON command for `emms-player-mpv-ipc-sentinel' to run when it connects to mpv.
+I.e. last command that either initiated connection or was used while
+connecting to mpv.
 Set by `emms-player-mpv-start' and such,
 cleared once it gets sent by `emms-player-mpv-ipc-sentinel'.")
 
 (defvar emms-player-mpv-ipc-id 1
-  "Auto-incremented value sent in JSON requests for request_id and observe_property id's.
-Use `emms-player-mpv-ipc-id-get' to get and increment this value, instead of using it directly.
-Wraps-around upon reaching `emms-player-mpv-ipc-id-max' (unlikely to ever happen).")
+  "Auto-incremented counter for unique JSON request identifiers.
+Use for for `request_id' and `observe_property' identifiers.
+Use `emms-player-mpv-ipc-id-get' to get and increment this value,
+instead of using it directly.
+Wraps-around upon reaching `emms-player-mpv-ipc-id-max'
+(unlikely to ever happen).")
 
 (defvar emms-player-mpv-ipc-id-max (expt 2 30)
   "Max value for `emms-player-mpv-ipc-id' to wrap around after.
-Should be fine with both mpv and emacs, and probably never reached anyway.")
+Should be fine with both mpv and Emacs, and probably never reached anyway.")
 
 (defvar emms-player-mpv-ipc-req-table nil
   "Auto-initialized hash table of outstanding API req_ids to their handler funcs.")
 
 (defvar emms-player-mpv-ipc-stop-command nil
   "Internal flag to track when stop command starts/finishes before next loadfile.
-Set to either nil, t or playback start function to call on end-file event after stop command.
-This is a workaround for mpv-0.30+ behavior, where 'stop + loadfile' only runs 'stop'.")
+Set to either nil, t or the playback start function to call on end-file event
+after stop command.
+This is a workaround for mpv-0.30+ behavior, where 'stop + loadfile' only
+runs 'stop'.")
 
 
 (defvar emms-player-mpv-event-connect-hook nil
-  "Normal hook run right after establishing new JSON IPC
-connection to mpv instance and before `emms-player-mpv-ipc-connect-command'	if any.
-Best place to send any observe_property, request_log_messages, enable_event commands.
+  "Normal hook run right after establishing new JSON IPC connection to mpv.
+Runs before `emms-player-mpv-ipc-connect-command', if any.
+Best place to send any `observe_property', `request_log_messages',
+`enable_event' commands.
 Use `emms-player-mpv-ipc-id-get' to get unique id values for these.
 See also `emms-player-mpv-event-functions'.")
 
@@ -246,8 +253,9 @@ to indicate that playback should stop instead of switching to next track.")
   (and (= emacs-major-version 26)
        (= emacs-minor-version 1))
   "Non-nil to enable workaround for issue #31901 in emacs 26.1.
-Emacs 26.1 fails to indicate missing socket file error for unix socket network processes
-that were started with :nowait t, so blocking connections are used there instead.")
+Emacs 26.1 fails to indicate missing socket file error for unix socket
+network processes that were started with :nowait t, so blocking connections
+are used there instead.")
 
 
 ;; ----- helpers
@@ -277,7 +285,7 @@ Strips whitespace from start/end of TPL-OR-MSG and strings in TPL-VALUES."
       (setq tpl-or-msg (replace-regexp-in-string "%" "%%" tpl-or-msg t t)))
     (let ((ts (float-time)))
       (unless emms-player-mpv-debug-ts-offset (setq emms-player-mpv-debug-ts-offset ts))
-      (apply 'message
+      (apply #'message
              (concat "emms-player-mpv %.1f " tpl-or-msg)
              (- ts emms-player-mpv-debug-ts-offset)
              tpl-values))))
@@ -562,24 +570,35 @@ HANDLER func will be called with decoded response JSON as (handler data err),
 where ERR will be either nil on \"success\", 'connection-error or whatever is in JSON.
 If HANDLER is nil, default `emms-player-mpv-ipc-req-error-printer'
 will be used to at least log errors.
+Multiple commands can be batched in one list as '(batch (cmd1 . handler1) ...),
+in which case common HANDLER argument is ignored.
 PROC can be specified to avoid `emms-player-mpv-ipc' call (e.g. from sentinel/filter funcs)."
-  (let
-      ((req-id (emms-player-mpv-ipc-id-get))
-       (req-proc (or proc (emms-player-mpv-ipc)))
-       (handler (or handler #'emms-player-mpv-ipc-req-error-printer)))
-    (unless emms-player-mpv-ipc-req-table
-      (setq emms-player-mpv-ipc-req-table (make-hash-table)))
-    (let ((json (concat (json-encode (list :command cmd :request_id req-id))
-                        "\n")))
-      (emms-player-mpv-debug-msg "json >> %s" json)
-      (condition-case _err
-          ;; On any disconnect, assume that mpv process is to blame and force restart.
-          (process-send-string req-proc json)
-        (error
-         (emms-player-mpv-proc-stop)
-         (funcall handler nil 'connection-error)
-         (setq handler nil))))
-    (when handler (puthash req-id handler emms-player-mpv-ipc-req-table))))
+  (dolist
+      (cmd-and-handler
+       (if (and (listp cmd)
+                (eq (car cmd)
+                    'batch))
+           (cdr cmd)
+         `((,cmd . ,handler))))
+    (cl-destructuring-bind (cmd . handler)
+        cmd-and-handler
+      (let
+          ((req-id (emms-player-mpv-ipc-id-get))
+           (req-proc (or proc (emms-player-mpv-ipc)))
+           (handler (or handler #'emms-player-mpv-ipc-req-error-printer)))
+        (unless emms-player-mpv-ipc-req-table
+          (setq emms-player-mpv-ipc-req-table (make-hash-table)))
+        (let ((json (concat (json-encode (list :command cmd :request_id req-id))
+                            "\n")))
+          (emms-player-mpv-debug-msg "json >> %s" json)
+          (condition-case _err
+              ;; On any disconnect, assume that mpv process is to blame and force restart.
+              (process-send-string req-proc json)
+            (error
+             (emms-player-mpv-proc-stop)
+             (funcall handler nil 'connection-error)
+             (setq handler nil))))
+        (when handler (puthash req-id handler emms-player-mpv-ipc-req-table))))))
 
 (defun emms-player-mpv-ipc-req-resolve (req-id data err)
   "Run handler-func for specified req-id."
@@ -637,6 +656,12 @@ potential duplication if used for same properties from different functions."
   (emms-player-mpv-debug-msg "idle-check (stopped=%s)" emms-player-mpv-stopped)
   (unless emms-player-mpv-stopped (emms-player-stopped)))
 
+(defun emms-player-mpv-event-playing-time-sync ()
+  "Request and update `emms-playing-time' after playback seek/restart or unpause."
+  (emms-player-mpv-ipc-req-send '(get_property time-pos)
+                                #'(lambda (pos err)
+                                    (unless err (emms-playing-time-set pos)))))
+
 (defun emms-player-mpv-event-handler (json-data)
   "Handler for supported mpv events, including property changes.
 Called before `emms-player-mpv-event-functions' and does same thing as these hooks."
@@ -647,7 +672,17 @@ Called before `emms-player-mpv-event-functions' and does same thing as these hoo
      ;;	and don't correspond to actual playback state.
      (unless (emms-player-mpv-proc-playing-p)
        (emms-player-mpv-proc-playing t)
-       (emms-player-started emms-player-mpv)))
+       (emms-player-started emms-player-mpv))
+     (emms-player-mpv-event-playing-time-sync))
+    ("pause"
+     (unless emms-player-paused-p
+       (setq emms-player-paused-p t)
+       (run-hooks 'emms-player-paused-hook)))
+    ("unpause"
+     (emms-player-mpv-event-playing-time-sync)
+     (when emms-player-paused-p
+       (setq emms-player-paused-p nil)
+       (run-hooks 'emms-player-paused-hook)))
     ("end-file"
      (when (emms-player-mpv-proc-playing-p)
        (emms-player-mpv-proc-playing nil)
@@ -671,7 +706,8 @@ Called before `emms-player-mpv-event-functions' and does same thing as these hoo
 
 (defun emms-player-mpv-info-meta-connect-func ()
   "Hook function for `emms-player-mpv-event-connect-hook' to update metadata from mpv."
-  (emms-player-mpv-observe-property 'metadata))
+  (emms-player-mpv-observe-property 'metadata)
+  (emms-player-mpv-observe-property 'duration))
 
 (defun emms-player-mpv-info-meta-event-func (json-data)
   "Hook function for `emms-player-mpv-event-functions' to update metadata from mpv."
@@ -719,32 +755,14 @@ Called before `emms-player-mpv-event-functions' and does same thing as these hoo
 (defun emms-player-mpv-info-duration-event-func (json-data)
   "Hook function for `emms-player-mpv-event-functions' to update track duration from mpv."
   (when
-      (string= (alist-get 'event json-data)
-               "playback-restart")
-    (emms-player-mpv-info-duration-check)))
-
-(defun emms-player-mpv-info-duration-check ()
-  "Check whether current mpv track has reliable duration info and request it."
-  (emms-player-mpv-ipc-req-send '(get_property stream-end)
-                                (lambda (pts-end err)
-                                  (if err
-                                      (unless (and (stringp err)
-                                                   (string= err "property unavailable"))
-                                        (emms-player-mpv-ipc-req-error-printer pts-end err))
-                                    (when pts-end
-                                      (emms-player-mpv-ipc-req-send '(get_property duration)
-                                                                    #'emms-player-mpv-info-duration-handler))))))
-
-(defun emms-player-mpv-info-duration-handler (duration err)
-  "Duration property request handler to update it for current emms track."
-  (if err
-      (emms-player-mpv-debug-msg "duration-req-error: %s" err)
-    ;; Duration can be nil or 0 for network streams, depending on version/stream
-    (when (and (numberp duration)
-               (> duration 0))
-      (let
-          ((duration (round duration))
-           (track (emms-playlist-current-selected-track)))
+      (and
+       (string= (alist-get 'event json-data) "property-change")
+       (string= (alist-get 'name json-data) "duration"))
+    (let
+        ((duration (alist-get 'data json-data))
+         (track (emms-playlist-current-selected-track)))
+      (when (and track (numberp duration) (> duration 0))
+        (setq duration (round duration))
         (emms-track-set track 'info-playing-time duration)
         (emms-track-set track 'info-playing-time-min (/ duration 60))
         (emms-track-set track 'info-playing-time-sec (% duration 60))))))
@@ -755,7 +773,9 @@ Called before `emms-player-mpv-event-functions' and does same thing as these hoo
 (defun emms-player-mpv-cmd (cmd &optional handler)
   "Send mpv command to process/connection if both are running,
 or otherwise schedule start/connect and set
-`emms-player-mpv-ipc-start-track' for `emms-player-mpv-ipc-sentinel'."
+`emms-player-mpv-ipc-connect-command' for `emms-player-mpv-ipc-sentinel'.
+Multiple commands can be batched in one list as '(batch (cmd1 . handler1) ...),
+in which case common HANDLER argument is ignored."
   (setq emms-player-mpv-ipc-connect-command nil)
   (let ((proc (emms-player-mpv-ipc)))
     (if proc
@@ -786,17 +806,6 @@ and will be removed in a future EMMS version."
   (memq (emms-track-type track)
         '(file url streamlist playlist)))
 
-(defun emms-player-mpv-start-error-handler (mpv-cmd _mpv-data mpv-error)
-  "Playback-restart error handler for `emms-player-mpv-cmd',
-to restart/reconnect-to mpv and re-run MPV-CMD,
-if there was any issue when trying to start it initially."
-  (if (eq mpv-error 'connection-error)
-      ;; Reconnect and restart playback if current connection fails (e.g. mpv crash)
-      (emms-player-mpv-cmd mpv-cmd
-                           (lambda (_mpv-data _mpv-error)
-                             (emms-player-mpv-cmd mpv-cmd)
-                             (emms-player-mpv-cmd `(set pause no))))))
-
 (defun emms-player-mpv-start (track)
   (setq emms-player-mpv-stopped nil)
   (emms-player-mpv-proc-playing nil)
@@ -812,11 +821,17 @@ if there was any issue when trying to start it initially."
                                      track-name)
           (emms-player-started emms-player-mpv))
       (let*
-          ((start-cmd (list (if track-is-playlist 'loadlist 'loadfile)
-                            track-name 'replace))
-           (start-func `(lambda ()
-                          (emms-player-mpv-cmd ',start-cmd
-                                               (apply-partially 'emms-player-mpv-start-error-handler ',start-cmd)))))
+          ((play-cmd
+            `(batch
+              ((,(if track-is-playlist 'loadlist 'loadfile)
+                ,track-name replace))
+              ((set pause no))))
+           (start-func
+            ;; Try running play-cmd and retry it on connection failure, e.g. if mpv died
+            (apply-partially 'emms-player-mpv-cmd play-cmd
+                             (lambda (_mpv-data mpv-error)
+                               (when (eq mpv-error 'connection-error)
+                                 (emms-player-mpv-cmd play-cmd))))))
         (if emms-player-mpv-ipc-stop-command
             (setq emms-player-mpv-ipc-stop-command start-func)
           (funcall start-func))))))
