@@ -1,14 +1,13 @@
 ;;; emms.el --- The Emacs Multimedia System  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008,
-;;   2009, 2018, 2020  Free Software Foundation, Inc.
+;; Copyright (C) 2003-2021  Free Software Foundation, Inc.
 
 ;; Author: Jorgen Schäfer <forcer@forcix.cx>, the Emms developers (see AUTHORS file)
 ;; Maintainer: Yoni Rabkin <yrk@gnu.org>
-;; Version: 7.0
+;; Version: 7.8
 ;; Keywords: emms, mp3, ogg, flac, music, mpeg, video, multimedia
 ;; Package-Type: multi
-;; Package-Requires: ((cl-lib "0.5") (seq))
+;; Package-Requires: ((cl-lib "0.5") (nadvice "0.3") (seq))
 ;; url: https://www.gnu.org/software/emms/
 
 ;; This file is part of EMMS.
@@ -45,7 +44,7 @@
 
 ;;; Code:
 
-(defvar emms-version "6.03"
+(defvar emms-version "7.8"
   "EMMS version string.")
 
 
@@ -68,7 +67,7 @@
   :group 'emms)
 
 (defcustom emms-player-list nil
-  "*List of players that EMMS can use.  You need to set this!"
+  "*List of players that EMMS can use."
   :group 'emms
   :type '(repeat (symbol :tag "Player")))
 
@@ -315,6 +314,10 @@ being the playlist buffer.")
 (defvar emms-playlist-buffer nil
   "The current playlist buffer, if any.")
 
+(defvar emms-players-preference-f #'emms-players-default-preference-f
+  "Default function for player preference.")
+
+
 
 ;;; Macros
 
@@ -537,9 +540,9 @@ See `emms-random-playlist'."
   (interactive)
   (setq emms-random-playlist (not emms-random-playlist))
   (if emms-random-playlist
-      (progn (setq emms-player-next-function 'emms-random)
+      (progn (setq emms-player-next-function #'emms-random)
              (message "Will play the tracks randomly."))
-    (setq emms-player-next-function 'emms-next-noerror)
+    (setq emms-player-next-function #'emms-next-noerror)
     (message "Will play the tracks sequentially.")))
 
 (defun emms-toggle-repeat-playlist ()
@@ -677,6 +680,10 @@ before the error is signaled."
   "Return the type of TRACK."
   (emms-track-get track 'type))
 
+(defun emms-track-file-p (track)
+  "True if TRACK is a file type"
+  (eq 'file (emms-track-type track)))
+
 (defun emms-track-name (track)
   "Return the name of TRACK."
   (emms-track-get track 'name))
@@ -1117,7 +1124,7 @@ This uses `emms-playlist-update-track-function'."
   "Insert tracks from SOURCE in the current playlist.
 This is supplying ARGS as arguments to the source."
   (with-current-emms-playlist
-    (apply 'emms-playlist-insert-source source args)))
+    (apply #'emms-playlist-insert-source source args)))
 
 (defun emms-playlist-tracks-in-region (beg end)
   "Return all tracks between BEG and END."
@@ -1213,7 +1220,7 @@ ignore this."
     (delete-region (point-min)
                    (point-max))
     (run-hooks 'emms-playlist-cleared-hook)
-    (mapc 'emms-playlist-insert-track
+    (mapc #'emms-playlist-insert-track
           (sort tracks emms-sort-lessp-function))
     (let ((pos (text-property-any (point-min)
                                   (point-max)
@@ -1249,7 +1256,7 @@ ignore this."
                                                 (point-max))))
     (delete-region (point-min) (point-max))
     (run-hooks 'emms-playlist-cleared-hook)
-    (mapc 'emms-playlist-insert-track
+    (mapc #'emms-playlist-insert-track
           (nreverse
            (emms-uniq-list tracks 'emms-track-name)))
     (let ((pos (text-property-any (point-min)
@@ -1357,7 +1364,7 @@ See emms-source-file.el for some examples."
   "Play the tracks of SOURCE, after first clearing the EMMS playlist."
   (emms-stop)
   (emms-playlist-current-clear)
-  (apply 'emms-playlist-current-insert-source source args)
+  (apply #'emms-playlist-current-insert-source source args)
   (emms-playlist-current-select-first)
   (emms-start))
 
@@ -1366,7 +1373,7 @@ See emms-source-file.el for some examples."
   (with-current-emms-playlist
     (save-excursion
       (goto-char (point-max))
-      (apply 'emms-playlist-current-insert-source source args))
+      (apply #'emms-playlist-current-insert-source source args))
     (when (or (not emms-playlist-selected-marker)
               (not (marker-position emms-playlist-selected-marker)))
       (emms-playlist-select-first))))
@@ -1375,7 +1382,7 @@ See emms-source-file.el for some examples."
   "Insert the tracks from SOURCE in the current buffer."
   (if (not emms-playlist-buffer-p)
       (error "Not in an EMMS playlist buffer")
-    (apply 'emms-playlist-insert-source source args)))
+    (apply #'emms-playlist-insert-source source args)))
 
 ;;; User-defined playlists
 ;;; FIXME: Shuffle is bogus here! (because of narrowing)
@@ -1451,11 +1458,43 @@ non-nil, or nil if no such player exists."
         (car lis)
       nil)))
 
+(defun emms-players-default-preference-f (track players)
+  "Default preference function.
+
+Returns the first player."
+  (ignore track)
+  (car players))
+
+(defun emms-players-preference (track players)
+  "Call `emms-players-preference-f' with TRACK and PLAYERS.
+
+The function `emms-players-preference-f' must accept an Emms
+track and a list of players. It can be assumed that all of the
+players in PLAYERS can play TRACK.
+
+The function must return one of the players from PLAYERS."
+  (funcall emms-players-preference-f track players))
+
+(defun emms-players-for (track)
+  "Return a player for TRACK.
+
+If the track can be played by more than one player, call
+`emms-players-preference' to choose a player."
+  (let (players)
+    (mapc
+     #'(lambda (player)
+	 (when (funcall (emms-player-get player 'playablep) track)
+	   (push player players)))
+     emms-player-list)
+    (if (< 1 (length players))
+	(emms-players-preference track players)
+      (car players))))
+
 (defun emms-player-start (track)
   "Start playing TRACK."
   (if emms-player-playing-p
       (error "A player is already playing")
-    (let ((player (emms-player-for track)))
+    (let ((player (emms-players-for track)))
       (if (not player)
           (error "Don't know how to play track: %S" track)
         ;; Change default-directory so we don't accidentally block any
